package com.myjavatools.xml;

import junit.framework.*;
import java.util.*;
import java.net.URL;
import java.io.File;

public class TestRss_0_92 extends TestCase {
  private Rss localSampleRss = null;
  private Rss remoteSampleRss = null;

  protected void setUp() throws Exception {
    super.setUp();
    try {
      remoteSampleRss = new Rss(new URL("http://cyber.law.harvard.edu/blogs/gems/tech/sampleRss092.xml"));
      remoteSampleRss.save("sample_rss_092_output.xml");
    } catch (Exception e)
    {
      // oops, no connection?
    }
    File source = new File("rss.v.0.91.xml");
    localSampleRss = new Rss(source);
  }

  protected void tearDown() throws Exception {
    localSampleRss = null;
    super.tearDown();
  }

  public void testGetDescription() {
    String expectedReturn ="A high-fidelity Grateful Dead song every day. This is where we're experimenting with enclosures on RSS news items that download when you're not using your computer. If it works (it will) it will be the end of the Click-And-Wait multimedia experience on the Internet. ";
    String actualReturn = remoteSampleRss.getDescription();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testGetDocs() {
    String expectedReturn = "http://backend.userland.com/rss092";
    String actualReturn = remoteSampleRss.getDocs();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testGetImage() {
    Rss.Image expectedReturn = null;
    Rss.Image actualReturn = remoteSampleRss.getImage();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testGetItems() {
    Rss.Item item1 = new Rss.Item("It's been a few days since I added a song to the Grateful Dead channel. Now that there are all these new Radio users, many of whom are tuned into this channel (it's #16 on the hotlist of upstreaming Radio users, there's no way of knowing how many non-upstreaming users are subscribing, have to do something about this..). Anyway, tonight's song is a live version of Weather Report Suite from Dick's Picks Volume 7. It's wistful music. Of course a beautiful song, oft-quoted here on Scripting News. <i>A little change, the wind and rain.</i>\n");
    item1.setEnclosure("http://www.scripting.com/mp3s/weatherReportDicksPicsVol7.mp3",
                       "6182912", "audio/mpeg");

    Rss.Item item2 = new Rss.Item("Kevin Drennan started a <a href=\"http://deadend.editthispage.com/\">Grateful Dead Weblog</a>. Hey it's cool, he even has a <a href=\"http://deadend.editthispage.com/directory/61\">directory</a>. <i>A Frontier 7 feature.</i>");

    item2.setSource("http://scriptingnews.userland.com/xml/scriptingNews2.xml", "Scripting News");

    Collection actualReturn = remoteSampleRss.getItems();
    assertEquals(22, actualReturn.size());
    assertEquals(item1, remoteSampleRss.findItemByDescription(item1.getDescription()));
    assertEquals(item1, remoteSampleRss.findItemByUrl(item1.getEnclosureUrl()));
    assertEquals(item2, remoteSampleRss.findItemByDescription(item2.getDescription()));
    assertEquals(item2, remoteSampleRss.findItemByUrl(item2.getSourceUrl()));
  }

  public void testGetLanguage() {
    String expectedReturn = "en-us";
    String actualReturn = localSampleRss.getLanguage();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testGetLastBuildDate() {
    String expectedReturn = null;
    String actualReturn = localSampleRss.getLastBuildDate();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testGetLink() {
    String expectedReturn = "http://www.xml.com/";
    String actualReturn = localSampleRss.getLink();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testSetLink() {
    String expectedReturn = "http://www.godzilla.org";
    localSampleRss.setLink(expectedReturn);
    String actualReturn = localSampleRss.getLink();
    assertEquals("return value", expectedReturn, actualReturn);    /**@todo fill in the test code*/
  }

  public void testGetManagingEditor() {
    String expectedReturn = "dave@userland.com (Dave Winer)";
    String actualReturn = remoteSampleRss.getManagingEditor();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testGetPubDate() {
    String expectedReturn = null;
    String actualReturn = localSampleRss.getPubDate();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testGetRating() {
    String expectedReturn = null;
    String actualReturn = localSampleRss.getRating();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testGetSkipDays() {
    String expectedReturn = null;
    String actualReturn = localSampleRss.getSkipDays();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testGetSkipHours() {
    String expectedReturn = null;
    String actualReturn = localSampleRss.getSkipHours();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testGetTextInput() {
    Rss.TextInput expectedReturn = null;
    Rss.TextInput actualReturn = localSampleRss.getTextInput();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testGetTitle() {
    String expectedReturn = "XML.com";
    String actualReturn = localSampleRss.getTitle();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testGetVersion() {
    String expectedReturn = "0.92";
    String actualReturn = remoteSampleRss.getOriginalVersion();
    assertEquals("return value", expectedReturn, actualReturn);
  }

  public void testGetWebMaster() {
    String expectedReturn = "dave@userland.com (Dave Winer)";
    String actualReturn = remoteSampleRss.getWebMaster();
    assertEquals("return value", expectedReturn, actualReturn);
  }
}
